using UnityEngine;
using UnityEngine.XR;
#if XR_CORE_UTILS
using Unity.XR.CoreUtils;
#endif

namespace YVR.Core
{
    public class CylinderShapeHandler : ILayerShapeHandler
    {
        public void HandleLayerPose(IYVRLayerHandle layerHandle, params object[] data)
        {
            int renderLayerId = (int) data[0];
            Transform transform = data[1] as Transform;
            YVRManager yvrManager = data[2] as YVRManager;
            float angle = (float) data[3];

            float radius = CalculateRadius(angle, transform.lossyScale.x);

            Vector3 origin = transform.position - (transform.forward * radius);
            XRPose originPose = new XRPose() {orientation = transform.rotation, position = origin};

            XRPose xrHeadPose = XRPose.identity;
            YVRCameraRig.centerEyeDevice.TryGetFeatureValue(CommonUsages.centerEyePosition, out xrHeadPose.position);
            YVRCameraRig.centerEyeDevice.TryGetFeatureValue(CommonUsages.centerEyeRotation, out xrHeadPose.orientation);

            XRPose pose = new XRPose();

#if XR_CORE_UTILS
        if (GameObject.FindObjectOfType<XROrigin>() != null)
        {
            pose =
 xrHeadPose * GameObject.FindObjectOfType<XROrigin>().Camera.transform.ToYVRPose().Inverse() * originPose;
        }
        else
        {
            pose = xrHeadPose * yvrManager.cameraRenderer.centerEyeCamera.transform.ToYVRPose().Inverse() * originPose;
        }
#else
            pose = xrHeadPose * yvrManager.cameraRenderer.centerEyeCamera.transform.ToYVRPose().Inverse() * originPose;
#endif

            layerHandle.SetLayerPose(renderLayerId, pose);
        }

        public void HandleLayerShape(IYVRLayerHandle layerHandle, params object[] data)
        {
            int renderLayerId = (int) data[0];
            Transform transform = data[1] as Transform;
            float angle = (float) data[2];

            float radius = CalculateRadius(angle, transform.lossyScale.x);
            float ratio = transform.lossyScale.x / transform.lossyScale.y;

            layerHandle.SetLayerCylinderParam(renderLayerId, radius, angle * Mathf.Deg2Rad, ratio);
        }

        private float CalculateRadius(float angle, float arcLength)
        {
            if (angle <= 0) return arcLength / 2f;
            return arcLength / ((2 * Mathf.PI) * (angle / 360f));
        }
    }
}